///////////////////////////////////////////////////
// nauty graph - funcion modified for our purpose
//      this file is included directly into ramsey/ramsey3.cpp
//      It contains function of the nauty package, that cannot be linked (static)
//           or required small changes described in function header  
///////////////////////////////////////////////////
static void encodegraphsize(int n, char **pp)   //rewritten as original is static as well 
/* Encode the size n in a string starting at **p, and reset **p
   to point to the character after the size */
{
    char *p;

    p = *pp;
    if (n <= SMALLN) 
        *p++ = BIAS6 + n;
    else if (n <= SMALLISHN)
    {
        *p++ = MAXBYTE;
        *p++ = BIAS6 + (n >> 12);
        *p++ = BIAS6 + ((n >> 6) & C6MASK);
        *p++ = BIAS6 + (n & C6MASK);
    }
    else 
    {
        *p++ = MAXBYTE;
        *p++ = MAXBYTE;
        *p++ = BIAS6 + (n >> 30);
        *p++ = BIAS6 + ((n >> 24) & C6MASK);
        *p++ = BIAS6 + ((n >> 18) & C6MASK);
        *p++ = BIAS6 + ((n >> 12) & C6MASK);
        *p++ = BIAS6 + ((n >> 6) & C6MASK);
        *p++ = BIAS6 + (n & C6MASK);
    }
    *pp = p;
}

char* pg_getline(FILE *f)     //rewritten according to name confict 
/* read a line with error checking */
/* includes \0.  Immediate EOF causes NULL return. */
{
    DYNALLSTAT(char,s,s_sz);
    int c;
    long i;

    DYNALLOC1(char,s,s_sz,5000,"getline");

    FLOCKFILE(f);
    i = 0;
    while ((c = GETC(f)) != EOF && c != '\n')
    {
        if (i == s_sz-3)
            DYNREALLOC(char,s,s_sz,3*(s_sz/2)+10000,"getline");
        s[i++] = c;
    }
    FUNLOCKFILE(f);

    if (i == 0 && c == EOF) return NULL;

    s[i] = '\0';
    return s;
}

graph* pg_readg(FILE *f, graph *g, int reqm, int *pm, int *pn) // original function didn't allowed additional characters (eg. description) in a line of a graph file 
/* read graph into nauty format */
{
    char *s,*p;
    int m,n;

    if ((pg_readg_line = pg_getline(f)) == NULL) return NULL;

    s = pg_readg_line;
    if (s[0] == ':')
    {
        readg_code = SPARSE6;
        p = s + 1;
    }
    else
    {
        readg_code = GRAPH6;
        p = s;
    }

    while (*p >= BIAS6 && *p <= MAXBYTE) 
        ++p;
    description = NULL; // pg - auxilliary variable to contain description of graph edges
    if (*p == ' ') description = p+1;   // pg - description of edges after a space 
    else if (*p != '\0')
        gt_abort(">E readg: illegal character\n");

    n = graphsize(s);
    if (readg_code == GRAPH6 && p - s != G6LEN(n))
        gt_abort(">E readg: truncated graph6 line\n");

    if (reqm > 0 && TIMESWORDSIZE(reqm) < n)
        gt_abort(">E readg: reqm too small\n");
    else if (reqm > 0)
        m = reqm;
    else
        m = (n + WORDSIZE - 1) / WORDSIZE;

    if (g == NULL)
    {
        if ((g = (graph*)ALLOCS(n,m*sizeof(graph))) == NULL)
            gt_abort(">E readg: malloc failed\n");
    }

    *pn = n;
    *pm = m;

    stringtograph(s,g,m);
    return g;
}

char* pg_ntog6(graph *g, int m, int n)  // original added "\n" and returned pointer to a static variable                      
/* convert nauty graph to graph6 string, including \0 */
{
    int i,j,k;
    char *p,x;
    set *gj;
    size_t ii = G6LEN(n)+2;

    char *gcode = (char*) malloc(ii);

    p = gcode;
    encodegraphsize(n,&p);

    k = 6;
    x = 0;

    for (j = 1; j < n; ++j)
    {
        gj = GRAPHROW(g,j,m);
        for (i = 0; i < j; ++i)
        {
            x <<= 1;
            if (ISELEMENT(gj,i)) x |= 1;
            if (--k == 0)
            {
                *p++ = BIAS6 + x;
                k = 6;
                x = 0;
            }
        }
    }
    if (k != 6) *p++ = BIAS6 + (x << k);
    *p = '\0';

    return gcode;
}

