// pgsort - tool for sorting files containing string of characters (via strcmp) 

#define USAGE \
  "pgsort [-s size] [-S mom] [-u] [-m pref start stop] [-o result] [file]\n       [--help]\n"

#define HELPTEXT \
  "Tool for sorting files containing string of characters (via strcmp)\n\
     -s size            : max string size (default value 100)\n\
     -S mom             : amount of memory, in megabits, for each block (default value 768)\n\
     -u                 : remove repetitions\n\
     -m pref start stop : merging sorted files pref_start..pref_stop\n\
     -o result          : output file (default stdout)\n\
      file              : input file (default stdin)\n"

#define SIZE 2048
#define MEMORY 768
#define REST 9

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>

#pragma warning (disable : 4996)

char ** buffer;
FILE ** files;
char ** first;
int * indeks;
int nr, unique;
FILE *input;
FILE *output;


void go_up(int i){
  int j;
  while (i) {
    j = (i-1)/2;
    
    if (strcmp(first[indeks[i]], first[indeks[j]]) < 0) { // i -th smaller - swap and continue up 
      int tmp = indeks[i];
      indeks[i] = indeks[j];
      indeks[j] = tmp;
      i = j;
    }
    else break;
  }
}

void go_down(int i) {
  int j = 2*i+1;

  while (j < nr) {
    if (j+1 < nr && strcmp(first[indeks[j]], first[indeks[j+1]]) > 0) j++; // j+1 smaller - change j
    
    // j - element to compare 
    if (strcmp(first[indeks[i]], first[indeks[j]]) > 0) { // i -th larger - swap and continue down 
      int tmp = indeks[i];
      indeks[i] = indeks[j];
      indeks[j] = tmp;
      i = j;
      j = 2*i+1;
    }
    else break;
  }
}

void quicksort(long d, long g) {
  // sorting buffer[d..g-1]
  if (g - d > REST) {
    long i, s;
    char *t, *t2;
    
    // swap the first and the middle element (not random!)
    i = (d+g)/2;
    t = buffer[i];
    buffer[i] = buffer[d];
    buffer[d] = t;

    // finding element not smaller than t
    s = d+1;
    while(s < g && strcmp(buffer[s], t) < 0) s++;
    
    // after decreasing s by one buffer[d+1..s] < t
    // check the rest
    for (i = s--; i < g; i++)
      if (strcmp(buffer[i], t) < 0) {
        // increase s and change 
        t2 = buffer[++s];
        buffer[s] = buffer[i];
        buffer[i] = t2;
      }
    // swap buffer[d] and buffer[s]
    buffer[d] = buffer[s];
    buffer[s] = t;
    // now buffer[d..s-1] < buffer[s] < buffer[s+1..g-1]
      
    quicksort(d, s); //sortuj [d..s-1]
    quicksort(s+1, g); //sortuj [s+1..g-1]
  }
}

void insertionsort(long d, long g) {
  long i, j;
  char *t;

  for (i = d+1; i < g; i++) {
    t = buffer[i];
    j = i;
    while (j > 0 && strcmp(t, buffer[j-1]) < 0) {
      buffer[j] = buffer[j-1];
      j--;
    }
    buffer[j] = t;
  }
}

int main(int argc, char * argv[])
{
  char prefiks[100], fname[100];
  long start = 0;
  long stop = 0;
  long i; 
  long size = SIZE;
  long mem = MEMORY;
  char *arg;
  int tmp = 0;
  char *last = new char[size];

  unique = 0;
  input = stdin;
  output = stdout;

  if (argc == 1) {
    printf(USAGE);
    return 0;
  }
  else for (i = 1; i < argc; i++) {
    arg = argv[i];
    if (*arg == '-') switch (*(++arg)) {
      case 's' : sscanf(argv[++i], "%d", &size); size++; break;
      case 'S' : sscanf(argv[++i], "%d", &mem); break;
      case 'u' : unique = 1; break;
      case 'm' : strcpy(prefiks, argv[++i]);
                 start = atol(argv[++i]);
                 stop = atol(argv[++i]);
                 break;
      case 'o' : output = fopen(argv[++i], "wt"); break;
      case '-' : printf(USAGE); printf(HELPTEXT); return 0; 
      default  : printf(USAGE); return 0;
    }
    else input = fopen(argv[i], "rt");
  }

  if (!start && !stop) { // sorting
      // size
    int tmp_size = 8 * ((size+7)/8) + sizeof(buffer);
    long buff_size = 1024; 
    long count;
    FILE * tmp_file;
    buff_size = (buff_size * 1024 * mem)/tmp_size;
    buff_size = 8 * (buff_size/8);

    buffer = new char * [buff_size];

    count = 0;
    while (!feof(input)) {
      if (!tmp) buffer[count]= new char[size];
      fscanf(input, "%s\n", buffer[count++]);
      if (ferror(input)) {
        fprintf(stderr, "Error!\n");
        return -1;
      }
        
      if (count == buff_size) {
        // mixed sorting 
        quicksort(0,buff_size);
        insertionsort(0, buff_size);

        if (!tmp) {
          struct stat b;
          // temporary files are needed
          tmp = start = stop = 1;
          i = count = 0;
          tmp_file = NULL;
          strcpy(prefiks, "pgsAAAA");
          // initiating pseudo-random numbers generator
          srand(((unsigned)time(NULL))+((unsigned)clock()));
          do {
            // prefix, fname
            for (i = 3; i <7; i++)  
              prefiks[i] = (char)(rand() % 26) + 'A';
            sprintf(fname, "%s_%d", prefiks, stop);
            // if it cannot be done 512 times, then finish with error
            if (count++ == 511) return 1;
            // if there is no file fname, then crete it
            if (stat(fname, &b)) tmp_file = fopen(fname, "wt");
          } while(!tmp_file);
          fprintf(stderr, "pgsort: %s\n", prefiks);
        }
        else {
          sprintf(fname, "%s_%d", prefiks, ++stop);
          tmp_file = fopen(fname, "wt");
        }
        if (unique) {
          last[0] = 0;
          for (i = 0; i < buff_size; i++)
            if (strcmp(last,buffer[i])) { // if there is on, then ignore
              strcpy(last, buffer[i]);
              fprintf(tmp_file, "%s\n", last);
              if (ferror(tmp_file)) {
                fprintf(stderr, "Error!\n");
                return -1;
              }
            }
        }
        else for (i = 0; i < buff_size; i++) {
            fprintf(tmp_file, "%s\n", buffer[i]);
            if (ferror(tmp_file)) {
              fprintf(stderr, "Error!\n");
              return -1;
            }
        }
        fclose(tmp_file);
        count = 0;
      }
    }
    if (count) { // non-empty buffer - sort
      // mixed sorting 
      quicksort(0,count);
      insertionsort(0, count);
      
      if (tmp) { // output to temporary file or to standard output
        sprintf(fname, "%s_%d", prefiks, ++stop);
        tmp_file = fopen(fname, "wt");
      }
      else tmp_file = output;
      if (unique) {
        last[0] = 0;
        for (i = 0; i < count; i++)
          if (strcmp(last,buffer[i])) { // if there is on, then ignore
            strcpy(last, buffer[i]);
            fprintf(tmp_file, "%s\n", last);
            if (ferror(tmp_file)) {
              fprintf(stderr, "Error!\n");
              return -1;
            }
          }
      }
      else for (i = 0; i < count; i++) {
          fprintf(tmp_file, "%s\n", buffer[i]);
          if (ferror(tmp_file)) {
            fprintf(stderr, "Error!\n");
            return -1;
          }
      }
      if (tmp) fclose(tmp_file);
    }
    
    // cleaning 
    if (start) count = buff_size;
    for (i = count-1; i >= 0; i--)
      delete[] buffer[i];
    delete[] buffer;
  }

  if (start || stop) { // merging
    files = new FILE *[stop-start+1];  
    first = new char *[stop-start+1];
    indeks = new int[stop-start+1];
  
    for (i = 0; i <= stop-start; i++) {
      first[i] = new char[size];   
      sprintf(fname, "%s_%d", prefiks, i+start);
      files[i] = fopen(fname, "rt");
    }

    // sorting 
    // load one element from the file
    nr = 0;
    for (i = 0; i <= stop-start; i++) if (files[i]) {
      if (fscanf(files[i], "%s\n", first[i]) == 1) {
        indeks[nr++] = i;
        go_up(nr-1);
      }
      else if (ferror(files[i])) {
        fprintf(stderr, "Error!\n");
        return -1;
      }
    }

    // keep removing the smallest element, if there is anything 
    if (unique) {
      last[0] = 0;
      while (nr > 0) {
        i = indeks[0];
        if (strcmp(last,first[i])) { // ignore if already encountered 
          strcpy(last, first[i]);
          fprintf(output, "%s\n", last);
          if (ferror(output)) {
            fprintf(stderr, "Error!\n");
            return -1;
          }
        }
        if (fscanf(files[i], "%s\n", first[i]) != 1)  {
          if (ferror(files[i])) {
            fprintf(stderr, "Error!\n");
            return -1;
          }
           else indeks[0] = indeks[--nr];
        }
        go_down(0);        
      }
    }
    else {
      while (nr > 0) {
        i = indeks[0];
        fprintf(output, "%s\n", first[i]);
        if (ferror(output)) {
          fprintf(stderr, "Error!\n");
          return -1;
        }
        if (fscanf(files[i], "%s\n", first[i]) != 1) {
          if (ferror(files[i])) {
            fprintf(stderr, "Error!\n");
            return -1;
          }
           else indeks[0] = indeks[--nr];
        }
        go_down(0);        
      }
    }

    //cleaning 
    for (i = stop-start; i>=0; i--) {
      delete[] first[i];
      if (files[i]) fclose(files[i]); 
    }
    delete[] indeks;
    delete[] first;
    delete[] files;

    // temporary files
    if (tmp) {
      sprintf(fname, "rm %s_*", prefiks);
      system(fname);
      fprintf(stderr, "%s\n", fname);
    }
  }
  
  // cleaning - close all files that are open 
  if (fileno(input) != fileno(stdin)) fclose(input);
  if (fileno(output) != fileno(stdout)) fclose(output);
  delete[] last;

  return 0;
}
