///////////////////////////////////////////////////
// Ramsey On-line game
// Author: Przemek Gordinowicz 
///////////////////////////////////////////////////
// Program creates script R_*.sh that runs the main program "Ramsey" (3 colour version)
///////////////////////////////////////////////////
// Parameters: (option) prefix symmetric 
//            -m             - only merge
//            -e             - only shorten - removing existing configurations 
//            -d             - only description
//            -de            - only description with using short files 
//            -r nr pref2    - only run (with partitioning into nr of pieces) writes to pref2* (merging on red and blue => non_green)
//            -rd nr pref2   - run on files with descriptions (with partitioning into nr of pieces)
//            -R nr pref2    - only Run reads from pref2* writes to prefix* (merging on non_green-green => NON_EDGE)
//            -Rd nr pref2   - only Run on files with description (a)
//            -Rde nr pref2  - only Run on files with description (a) and shorten (b) (non_green-green)
//            -u p_new       - "go up" - existing files will be names as run ones 
//                        with the number of allowed edges increased by 1 (it allows tafinishing 
//                        the algorithm with a smaller bound (after removing some files) 
//                        the name will contain a new prefix
///////////////////////////////////////////////////
// Browse the file prefiks.txt for info about files with a particular prefix (encoding number of edges)
// written by Ramsey. Comparing number of coloured edges creates batch file with list of calls  
// for the next iteration of Ramsey program according to:
//    r_1*b_2 > 0 && max(r_1-1, max_r_2) + max(b_1, b_2-1) + max(g_1, g_2) < edge_no
// or
//    b_1*r_2 > 0 && max(r_1, max_r_2-1) + max(b_1-1, b_2) + max(g_1, g2) < edge_no
// and/or analogous when the symmetric Ramsey number is computed 
// When option -R* is used takes a file (1) with non_green edge and another (with green edge) and
// use the condition: 
//    g_2 > 0 && max(r_1, max_r_2) + max(b_1, b_2) + max(g_1, g_2-1) < edge_no
//////////////////////////////////////////////////
// Note: number of edges lower than 255!
/////////////////////////////////////////////////
// The rest of comments writen in Polish. 
/////////////////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <math.h>
#include "ramsey.h"

void quicksort(long *buffer, long d, long g) {
  // sortuje buffer[d..g-1]
  if (g - d > 1) {
    long i, s;
    long t, t2;  
    // zamiana pierwszego i środkowego (nie losowego!)
    i = (d+g)/2;
    t = buffer[i];
    buffer[i] = buffer[d];
    buffer[d] = t;
    // jałowy przebieg - znaleźc element nie mniejszy niż t
    s = d+1;
    while(s < g && buffer[s] < t) s++;    
    // po pomniejszeniu s o jeden buffer[d+1..s] < t
    // przeglądamy resztę
    for (i = s--; i < g; i++)
      if (buffer[i] < t) {
        // powiększenie s i zamiana
        t2 = buffer[++s];
        buffer[s] = buffer[i];
        buffer[i] = t2;
      }
    // zamiana buffer[d] z buffer[s]
    buffer[d] = buffer[s];
    buffer[s] = t;
    // teraz buffer[d..s-1] < buffer[s] < buffer[s+1..g-1]      
    quicksort(buffer, d, s); //sortuj [d..s-1]
    quicksort(buffer, s+1, g); //sortuj [s+1..g-1]
  }
}

int max(int a, int b) {
  return (a > b ? a : b);
}

int main(int argc, char * argv[]) {
  if (argc >= 2) {
      FILE *report;
      long liczby[20000];
      double fsize[20000];
      int max_kraw;
      int ile, i, j, n, m, w, x, y, z;
      double total_size, size, c_size, t;
      struct stat descript;
      int k, r, b, g;
      int _k, _r, _b, _g;
      int merge, run, Run, split, go_up, descr, shorten, amnesty;
      int good, symmetric;
      char *prefix, *p_new;
      char buf[200];
      char fname[50];
      char prefix2[20];
      char prefix3[20];

      // parse args 
      if (*argv[1] == '-') {
        merge = argv[1][1] == 'm';
        run = argv[1][1] == 'r';
        Run = argv[1][1] == 'R';
        go_up = argv[1][1] == 'u';
        shorten = argv[1][1] == 'e';
        descr = argv[1][1] == 'd';
        amnesty = argv[1][1] == 'a';
        i = 2;
        if (go_up) p_new = argv[i++];
        if (descr) {
          shorten = argv[1][2] == 'e';
        }
        if (run || Run) {
          descr = argv[1][2] == 'd';
          shorten = argv[1][2] == 'e';
          if (Run && descr) shorten |= argv[1][3] == 'e';
          split = atol(argv[i++]);
          p_new = argv[i++];
        }
      }
      else {
        printf("Usage: TODO\n");
        return 0;
      }
      prefix = argv[i++];
      symmetric = argc > i && atol(argv[i++]) == 1;

      if (go_up) {
        // zmiana raportu i nazw plików "o jedną krawędź w górę" TODO - pliki b, shorten i descr
        FILE *report_new;         
        sprintf(buf, "%s.txt", prefix);
        report = fopen(buf, "rt");
        sprintf(buf, "%s.txt", p_new);
        report_new = fopen(buf, "wt");
        while(!feof(report)) {
          fscanf(report, "%d_%d_%d_%d\n", &k, &r, &b, &g);
          fprintf(report_new, "%d_%d_%d_%d\n", k+1, r, b, g);
          sprintf(buf, "%s %s_%d_%d_%d_%d %s_%d_%d_%d_%d", mv_command, prefix, k, r, b, g, p_new, k+1, r, b, g);
          system(buf);
        }
        fclose(report_new);
        fclose(report);
      }
      else {
        if (merge) {
          // najpier sortowanie żeby usunąć powtórzenia
          sprintf(buf, "%s %s.txt %s.tx", mv_command, prefix, prefix);
          system(buf);
          sprintf(buf, "%s -u %s.tx >%s.txt", sort_command, prefix, prefix);
          system(buf);
          sprintf(buf, "%s %s.tx", del_command, prefix);
          system(buf);
        }

        sprintf(buf, "%s.txt", prefix);
        report = fopen(buf, "rt");
        for(ile = 0; !feof(report); ile++) {
          fscanf(report, "%d_%d_%d_%d\n", &k, &r, &b, &g);
          liczby[ile] = (k << 24) + (r << 16) + (b << 8) + g;
        }
        fclose(report);
        quicksort(liczby, 0, ile);

        max_kraw = liczby[0] >> 24;
        if (merge) {
          sprintf(buf, "%s_%dm.%s", prefix, max_kraw, batch_extension);
          FILE *batch= fopen(buf, "wt");      
          // merge
          for (i = 0; (i < ile) && (liczby[i] >> 24 == max_kraw); i++) {
             _g = liczby[i] & 255;
             _b = (liczby[i] >> 8) & 255;
             _r = (liczby[i] >> 16) & 255;

             fprintf(batch, "%s %s_%d_%d_%d_%d\n", merge_command, prefix, max_kraw, _r, _b, _g);
          }
          fclose(batch);
        }
        if (amnesty) {
          sprintf(buf, "%s_%da.%s", prefix, max_kraw, batch_extension);
          FILE *batch= fopen(buf, "wt");      
          // descript
          for (i = 0; i < ile; i++) {
             _g = liczby[i] & 255;
             _b = (liczby[i] >> 8) & 255;
             _r = (liczby[i] >> 16) & 255;
             _k = liczby[i] >> 24;

             fprintf(batch, "%s -a %s_%d_%d_%d_%d %d\n", ramsey_command, prefix, _k, _r, _b, _g, symmetric);
          }
          fclose(batch);
        }
        if (shorten && !descr && !run) {
          sprintf(buf, "%s_%de.%s", prefix, max_kraw, batch_extension);
          FILE *batch= fopen(buf, "wt");      

          for (i = 0; (i < ile) && (liczby[i] >> 24 == max_kraw); i++) {
             _g = liczby[i] & 255;
             _b = (liczby[i] >> 8) & 255;
             _r = (liczby[i] >> 16) & 255;

            fprintf(batch, "%s3a -e %s %s_%d_%d_%d_%d %d -n %s_%d_%d_%d_%d", ramsey_command, prefix, prefix, max_kraw, _r, _b, _g, symmetric, prefix, max_kraw, _r, _b, _g);

            for (j = i+1; j < ile; j++) {
              g = liczby[j] & 255;
              b = (liczby[j] >> 8) & 255;
              r = (liczby[j] >> 16) & 255;
              k = liczby[j] >> 24;
              if (g == _g && b == _b && r == _r) fprintf(batch, " %s_%d_%d_%d_%d", prefix, k, r, b, g);
            }
            fprintf(batch, " -n");
            fprintf(batch, " -r");
            for (j = i-1; j >= 0; j--) {
              g = liczby[j] & 255;
              b = (liczby[j] >> 8) & 255;
              r = (liczby[j] >> 16) & 255;
              if (g == _g && b == _b && r == _r-1) { 
                fprintf(batch, " %s_%d_%d_%d_%d", prefix, max_kraw, r, b, g);
                break;
              } 
              else if (r < _r-1 || (r == _r-1 && b < _b) || (r == r-1 && b == _b && g < _g)) break;
            }
            for (j = i+1; j < ile; j++) {
              g = liczby[j] & 255;
              b = (liczby[j] >> 8) & 255;
              r = (liczby[j] >> 16) & 255;
              k = liczby[j] >> 24;
              if (g == _g && b == _b && r == _r-1) fprintf(batch, " %s_%d_%d_%d_%d", prefix, k, r, b, g);
            }
            fprintf(batch, " -b");
            for (j = i-1; j >= 0; j--) {
              g = liczby[j] & 255;
              b = (liczby[j] >> 8) & 255;
              r = (liczby[j] >> 16) & 255;
              if (g == _g && b == _b-1 && r == _r) { 
                fprintf(batch, " %s_%d_%d_%d_%d", prefix, max_kraw, r, b, g);
                break;
              } 
              else if (r < _r || b < _b-1 || (b == _b-1 && g < _g)) break;
            }
            for (j = i+1; j < ile; j++) {
              g = liczby[j] & 255;
              b = (liczby[j] >> 8) & 255;
              r = (liczby[j] >> 16) & 255;
              k = liczby[j] >> 24;
              if (g == _g && b == _b-1 && r == _r) fprintf(batch, " %s_%d_%d_%d_%d", prefix, k, r, b, g);
            }
            fprintf(batch, " -g");
            if (i > 0 && liczby[i-1] == liczby[i]-1) fprintf(batch, " %s_%d_%d_%d_%d", prefix, max_kraw, _r, _b, _g-1);
            for (j = i+1; j < ile; j++) {
              g = liczby[j] & 255;
              b = (liczby[j] >> 8) & 255;
              r = (liczby[j] >> 16) & 255;
              k = liczby[j] >> 24;
              if (g == _g-1 && b == _b && r == _r) fprintf(batch, " %s_%d_%d_%d_%d", prefix, k, r, b, g);
            }
            fprintf(batch, "\n");
          }
          fclose(batch);
        }
        if (descr && !run && !Run) {
          if (shorten)
            sprintf(prefix2, "%s%s", prefix, short_extension);
          else strcpy(prefix2, prefix);
          sprintf(buf, "%s_%dd.%s", prefix, max_kraw, batch_extension);
          FILE *batch= fopen(buf, "wt");      
          // descript
          for (i = 0; (i < ile) && (liczby[i] >> 24 == max_kraw); i++) {
             _g = liczby[i] & 255;
             _b = (liczby[i] >> 8) & 255;
             _r = (liczby[i] >> 16) & 255;

             fprintf(batch, "%s3a %s %s_%d_%d_%d_%d\n", ramsey_command, prefix, prefix2, max_kraw, _r, _b, _g);
          }
          fclose(batch);
        }
        if (run || Run) { 
          if (descr)
            sprintf(prefix2, "%s%s", prefix, descr_extension);
          else if (shorten)
            sprintf(prefix2, "%s%s", prefix, short_extension);
          else strcpy(prefix2, prefix);
          if (Run && descr && shorten)
            sprintf(prefix3, "%s%s", p_new, short_extension);      
          else strcpy(prefix3, p_new);
          //rozmiary plików
          for (i =0; i < ile; i++) {
            g = liczby[i] & 255;
            b = (liczby[i] >> 8) & 255;
            r = (liczby[i] >> 16) & 255;
            k = liczby[i] >> 24;
            sprintf(buf, "%s_%d_%d_%d_%d", prefix2, k, r, b, g);
            if (stat(buf, &descript) == 0)
              fsize[i] = ((double)descript.st_size)/1024; //rozmiar pliku w KB
          }
        }
        if (run) {
          sprintf(buf, "%s_%d.%s", prefix, max_kraw, batch_extension);
          FILE *batch= fopen(buf, "wt");
          m = 0;
          total_size = 0;
          for (i = 0; (i < ile) && (liczby[i] >> 24 == max_kraw); i++) {
             _g = liczby[i] & 255;
             _b = (liczby[i] >> 8) & 255;
             _r = (liczby[i] >> 16) & 255;

             for (j = i; j < ile; j++) {
               g = liczby[j] & 255;
               b = (liczby[j] >> 8) & 255;
               r = (liczby[j] >> 16) & 255;
               k = liczby[j] >> 24;
                
               if (symmetric)
                 good = (_r*b > 0 && max(r, _r-1) + max(b-1, _b) + max(g, _g) < max_kraw) ||    // _r,_b,_g ; r,b,g
                        (_r*g > 0 && max(b, _r-1) + max(g-1, _b) + max(r, _g) < max_kraw) ||    // _r,_b,_g ; b,g,r
                        (_r*r > 0 && max(g, _r-1) + max(r-1, _b) + max(b, _g) < max_kraw) ||    // _r,_b,_g ; g,r,b
                        (_r*b > 0 && max(r, _r-1) + max(b-1, _g) + max(g, _b) < max_kraw) ||     // _r,_g,_b ; r,b,g
                        (_r*g > 0 && max(b, _r-1) + max(g-1, _g) + max(r, _b) < max_kraw) ||     // _r,_g,_b ; b,g,r
                        (_r*r > 0 && max(g, _r-1) + max(r-1, _g) + max(b, _b) < max_kraw) ||    // _r,_g,_b ; g,r,b
                        (_b*b > 0 && max(r, _b-1) + max(b-1, _r) + max(g, _g) < max_kraw) ||    // _b,_r,_g ; r,b,g
                        (_b*g > 0 && max(b, _b-1) + max(g-1, _r) + max(r, _g) < max_kraw) ||     // _b,_r,_g ; b,g,r
                        (_b*r > 0 && max(g, _b-1) + max(r-1, _r) + max(b, _g) < max_kraw) ||    // _b,_r,_g ; g,r,b
                        (_b*b > 0 && max(r, _b-1) + max(b-1, _g) + max(g, _r) < max_kraw) ||     // _b,_g,_r ; r,b,g
                        (_b*g > 0 && max(b, _b-1) + max(g-1, _g) + max(r, _r) < max_kraw) ||     // _b,_g,_r ; b,g,r
                        (_b*r > 0 && max(g, _b-1) + max(r-1, _g) + max(b, _r) < max_kraw) ||    // _b,_g,_r ; g,r,b
                        (_g*b > 0 && max(r, _g-1) + max(b-1, _r) + max(g, _b) < max_kraw) ||     // _g,_r,_b ; r,b,g
                        (_g*g > 0 && max(b, _g-1) + max(g-1, _r) + max(r, _b) < max_kraw) ||    // _g,_r,_b ; b,g,r
                        (_g*r > 0 && max(g, _g-1) + max(r-1, _r) + max(b, _b) < max_kraw) ||    // _g,_r,_b ; g,r,b
                        (_g*b > 0 && max(r, _g-1) + max(b-1, _b) + max(g, _r) < max_kraw) ||     // _g,_b,_r ; r,b,g
                        (_g*g > 0 && max(b, _g-1) + max(g-1, _b) + max(r, _r) < max_kraw) ||    // _g,_b,_r ; b,g,r
                        (_g*r > 0 && max(g, _g-1) + max(r-1, _b) + max(b, _r) < max_kraw);      // _g,_b,_r ; g,r,b
               else good = (_r*b > 0 && max(r, _r-1) + max(b-1, _b) + max(g, _g) < max_kraw) ||  // _r,_b,_g ; r,b,g
                           (r*_b > 0 && max(b, _b-1) + max(r-1, _r) + max(g, _g) < max_kraw);    // _b,_r,_g ; b,r,g

               if (good) {
                  m++;
                  total_size += fsize[i] * fsize[j];
                  fprintf(batch, "%s3a %d %s %d %s_%d_%d_%d_%d %s_%d_%d_%d_%d %d\n", ramsey_command, m, p_new, max_kraw-1, prefix2, max_kraw, _r, _b, _g, prefix2, k, r, b, g, symmetric);
               }
             }
          }
          fclose(batch);
          printf("Ramrun %s %d. Tasks: %d. Total size: %.1f\n", prefix, max_kraw, m, total_size);
          if (split > 1) { // trzeba podzielić pracę na split kawałków 
            m = 0; i = 0; j = 0;
            n = 0; 
            c_size = 0;
            size = total_size/(split-n); //rozmiar +/- zadania w n-tym skrypcie
            sprintf(buf, "%s_%d_%d.%s", prefix, max_kraw, n, batch_extension);
            FILE *batch= fopen(buf, "wt");
            
            for (i = 0; (i < ile) && (liczby[i] >> 24 == max_kraw); i++) {
               _g = liczby[i] & 255;
               _b = (liczby[i] >> 8) & 255;
               _r = (liczby[i] >> 16) & 255;

               for (j = i; j < ile; j++) {
                 g = liczby[j] & 255;
                 b = (liczby[j] >> 8) & 255;
                 r = (liczby[j] >> 16) & 255;
                 k = liczby[j] >> 24;
                  
                 if (symmetric)
                   good = (_r*b > 0 && max(r, _r-1) + max(b-1, _b) + max(g, _g) < max_kraw) ||    // _r,_b,_g ; r,b,g
                          (_r*g > 0 && max(b, _r-1) + max(g-1, _b) + max(r, _g) < max_kraw) ||    // _r,_b,_g ; b,g,r
                          (_r*r > 0 && max(g, _r-1) + max(r-1, _b) + max(b, _g) < max_kraw) ||    // _r,_b,_g ; g,r,b
                          (_r*b > 0 && max(r, _r-1) + max(b-1, _g) + max(g, _b) < max_kraw) ||     // _r,_g,_b ; r,b,g
                          (_r*g > 0 && max(b, _r-1) + max(g-1, _g) + max(r, _b) < max_kraw) ||     // _r,_g,_b ; b,g,r
                          (_r*r > 0 && max(g, _r-1) + max(r-1, _g) + max(b, _b) < max_kraw) ||    // _r,_g,_b ; g,r,b
                          (_b*b > 0 && max(r, _b-1) + max(b-1, _r) + max(g, _g) < max_kraw) ||    // _b,_r,_g ; r,b,g
                          (_b*g > 0 && max(b, _b-1) + max(g-1, _r) + max(r, _g) < max_kraw) ||     // _b,_r,_g ; b,g,r
                          (_b*r > 0 && max(g, _b-1) + max(r-1, _r) + max(b, _g) < max_kraw) ||    // _b,_r,_g ; g,r,b
                          (_b*b > 0 && max(r, _b-1) + max(b-1, _g) + max(g, _r) < max_kraw) ||     // _b,_g,_r ; r,b,g
                          (_b*g > 0 && max(b, _b-1) + max(g-1, _g) + max(r, _r) < max_kraw) ||     // _b,_g,_r ; b,g,r
                          (_b*r > 0 && max(g, _b-1) + max(r-1, _g) + max(b, _r) < max_kraw) ||    // _b,_g,_r ; g,r,b
                          (_g*b > 0 && max(r, _g-1) + max(b-1, _r) + max(g, _b) < max_kraw) ||     // _g,_r,_b ; r,b,g
                          (_g*g > 0 && max(b, _g-1) + max(g-1, _r) + max(r, _b) < max_kraw) ||    // _g,_r,_b ; b,g,r
                          (_g*r > 0 && max(g, _g-1) + max(r-1, _r) + max(b, _b) < max_kraw) ||    // _g,_r,_b ; g,r,b
                          (_g*b > 0 && max(r, _g-1) + max(b-1, _b) + max(g, _r) < max_kraw) ||     // _g,_b,_r ; r,b,g
                          (_g*g > 0 && max(b, _g-1) + max(g-1, _b) + max(r, _r) < max_kraw) ||    // _g,_b,_r ; b,g,r
                          (_g*r > 0 && max(g, _g-1) + max(r-1, _b) + max(b, _r) < max_kraw);      // _g,_b,_r ; g,r,b
                 else good = (_r*b > 0 && max(r, _r-1) + max(b-1, _b) + max(g, _g) < max_kraw) ||  // _r,_b,_g ; r,b,g
                             (r*_b > 0 && max(b, _b-1) + max(r-1, _r) + max(g, _g) < max_kraw);    // _b,_r,_g ; b,r,g

                 if (good) {
                    if (size < 3*fsize[i] * fsize[j]) {  // każdy plik ma co najmniej 3 zadania    
                      if (2*fsize[i] > sqrt(size)) {  // dzielę i-ty plik (nieco chętniej)
                        // split file[i], na x +/- równych kawałków
                        x = (int)ceil(3*fsize[i]/sqrt(size)); // jak już dzielę to na więcej kawałków
                        sprintf(fname, "%s_%d_%d_%d_%d", prefix2, max_kraw, _r, _b, _g);
                        sprintf(buf, "%s_0", fname);
                        if (stat(buf, &descript)) { // jeżeli nie było podziału to podziel
                          sprintf(buf, "%s -d -C %d %s %s_", split_command, (int)ceil(1024*fsize[i]/x+120), fname, fname);
                          system(buf);

                          z = x > 10 ? 9 : x - 1;  // nazwy jednocyfrowe
                          for ( ; z >= 0; z--) {
                            sprintf(buf, "%s %s_0%d %s_%d", mv_command, fname, z, fname, z);
                            system(buf);
                          }
                          printf("%d %s split into %d pieces.\n", m+1, fname, x);
                        }
                        else x = (int)ceil(fsize[i]*1024/descript.st_size);
                      }
                      else x = 1;
                      if (i != j && 2*fsize[j] > sqrt(size)) { //dzielę j-ty plik
                        // split file[i], na x +/- równych kawałków
                        y = (int)ceil(3*fsize[j]/sqrt(size)); // jak już dzielę to na więcej kawałków
                        sprintf(fname, "%s_%d_%d_%d_%d", prefix2, k, r, b, g);
                        sprintf(buf, "%s_0", fname);
                        if (stat(buf, &descript)) { // jeżeli nie było podziału to podziel
                          sprintf(buf, "%s -d -C %d %s %s_", split_command, (int)ceil(1024*fsize[j]/y+120), fname, fname);
                          system(buf);

                          z = y > 10 ? 9 : y - 1;  // nazwy jednocyfrowe
                          for ( ; z >= 0; z--) {
                            sprintf(buf, "%s %s_0%d %s_%d", mv_command, fname, z, fname, z);
                            system(buf);
                          }
                          printf("%d %s split into %d pieces.\n", m+1, fname, y);
                        }
                        else y = (int)ceil(fsize[j]*1024/descript.st_size);
                      }
                      else y = 1;
                      if (i == j) { // ta sama para - po x plików
                        t = fsize[i]*fsize[i]/x/x; // mniej więcej rozmiar jednego "kawałka"
                        for (y = 0; y < x; y++) {
                          total_size -= y * t;  // odliczam "za darmo"
                          for (z = y; z < x; z++) {
                            m++;
                            if (fabs(size-c_size) < fabs(size - c_size - t)) { // nowy plik
                              fclose(batch);
                              total_size -= c_size;        // tyle zostało do zapisania
                              printf("File no %d, up to: %d, work-size: %.1f\n", n, m-1, c_size);
                              n++; c_size = 0;
                              size = total_size/(split-n); //rozmiar +/- zadania w n-tym skrypcie
                              sprintf(buf, "%s_%d_%d.%s", prefix, max_kraw, n, batch_extension);
                              FILE *batch= fopen(buf, "wt");
                            }
                            c_size += t;
                            fprintf(batch, "%s3a %d %s %d %s_%d %s_%d %d\n", ramsey_command, m, p_new, max_kraw-1, fname, y, fname, z, symmetric);
                          }
                        }
                      }
                      else { // x - i-tych, y -j-tych plików
                        t = fsize[i]/x * fsize[j]/y;
                        for (w = 0; w < x; w++)
                          for (z = 0; z < y; z++) {
                            m++;
                            if (fabs(size-c_size) < fabs(size - c_size - t)) { // nowy plik
                              fclose(batch);
                              total_size -= c_size;        // tyle zostało do zapisania
                              printf("File no %d, up to: %d, work-size: %.1f\n", n, m-1, c_size);
                              n++; c_size = 0;
                              size = total_size/(split-n); //rozmiar +/- zadania w n-tym skrypcie
                              sprintf(buf, "%s_%d_%d.%s", prefix, max_kraw, n, batch_extension);
                              FILE *batch= fopen(buf, "wt");
                            }
                            c_size += t;
                            if (x == 1) fprintf(batch, "%s3a %d %s %d %s_%d_%d_%d_%d %s_%d %d\n", ramsey_command, m, p_new, max_kraw-1, prefix2, max_kraw, _r, _b, _g, fname, z, symmetric);
                            else if (y == 1) fprintf(batch, "%s3a %d %s %d %s_%d %s_%d_%d_%d_%d %d\n", ramsey_command, m, p_new, max_kraw-1, fname, w, prefix2, k, r, b, g, symmetric);
                            else fprintf(batch, "%s3a %d %s %d %s_%d_%d_%d_%d_%d %s_%d %d\n", ramsey_command, m, p_new, max_kraw-1, prefix2, max_kraw, _r, _b, _g, w, fname, z, symmetric);
                          }
                      }
                    }
                    else {
                      m++;
                      if (fabs(size-c_size) < fabs(size - c_size - fsize[i]*fsize[j])) { // nowy plik
                        fclose(batch);
                        total_size -= c_size;        // tyle zostało do zapisania
                        printf("File no %d, up to: %d, work-size: %.1f\n", n, m-1, c_size);
                        n++; c_size = 0;
                        size = total_size/(split-n); //rozmiar +/- zadania w n-tym skrypcie
                        sprintf(buf, "%s_%d_%d.%s", prefix, max_kraw, n, batch_extension);
                        FILE *batch= fopen(buf, "wt");
                      }
                      c_size += fsize[i] * fsize[j];
                      fprintf(batch, "%s3a %d %s %d %s_%d_%d_%d_%d %s_%d_%d_%d_%d %d\n", ramsey_command, m, p_new, max_kraw-1, prefix2, max_kraw, _r, _b, _g, prefix2, k, r, b, g, symmetric);
                    }
                 }
               }
            }
            fclose(batch);
            printf("File no %d, up to: %d, work-size: %.1f\n", n, m, c_size);
          } 
        }
        if (Run) { 
          long liczby2[20000];
          double fsize2[20000];
          int ile2;
          
          sprintf(buf, "%s.txt", p_new);
          report = fopen(buf, "rt");
          for(ile2 = 0; !feof(report); ile2++) {
            fscanf(report, "%d_%d_%d_%d\n", &k, &r, &b, &g);
            liczby2[ile2] = (k << 24) + (r << 16) + (b << 8) + g;
          }
          fclose(report);
          quicksort(liczby2, 0, ile2);

          //rozmiary plików
          for (i = 0; i < ile2; i++) {
            g = liczby2[i] & 255;
            b = (liczby2[i] >> 8) & 255;
            r = (liczby2[i] >> 16) & 255;
            k = liczby2[i] >> 24;
            sprintf(buf, "%s_%d_%d_%d_%d", prefix3, k, r, b, g);
            if (stat(buf, &descript) == 0)
              fsize2[i] = ((double)descript.st_size)/1024; //rozmiar pliku w KB
          }

          sprintf(buf, "%s_%d.%s", p_new, max_kraw-1, batch_extension);
          FILE *batch= fopen(buf, "wt");
          m = 0;
          total_size = 0;
          for (i = 0; (i < ile2) && (liczby2[i] >> 24 == max_kraw-1); i++) {
             _g = liczby2[i] & 255;
             _b = (liczby2[i] >> 8) & 255;
             _r = (liczby2[i] >> 16) & 255;

             for (j = 0; j < ile; j++) {
               g = liczby[j] & 255;
               b = (liczby[j] >> 8) & 255;
               r = (liczby[j] >> 16) & 255;
               k = liczby[j] >> 24;

               good = g > 0 && max(r, _r) + max(b, _b) + max(g-1, _g) < max_kraw;

               if (symmetric)
                 good = good || (g > 0 && max(b, _r) + max(r, _b) + max(g-1, _g) < max_kraw) ||    // b,r,g
                                (b > 0 && max(r, _r) + max(g, _b) + max(b-1, _g) < max_kraw) ||   // r,g,b
                                (r > 0 && max(b, _r) + max(g, _b) + max(r-1, _g) < max_kraw) ||   // b,g,r
                                (b > 0 && max(g, _r) + max(r, _b) + max(b-1, _g) < max_kraw) ||   // g,r,b
                                (r > 0 && max(g, _r) + max(b, _b) + max(r-1, _g) < max_kraw);      // g,b,r
               if (good) {
                  m++;
                  total_size += fsize2[i] * fsize[j];
                  fprintf(batch, "%s3b %d %s %d %s_%d_%d_%d_%d %s_%d_%d_%d_%d %d\n", ramsey_command, m, prefix, max_kraw-1, prefix3, max_kraw-1, _r, _b, _g, prefix2, k, r, b, g, symmetric);
               }
             }
          }
          // i pozostaje takie samo
          for ( ; i < ile2; i++) {
             _g = liczby2[i] & 255;
             _b = (liczby2[i] >> 8) & 255;
             _r = (liczby2[i] >> 16) & 255;
             _k = liczby2[i] >> 24;

             for (j = 0; j < ile && (liczby[j] >> 24 == max_kraw); j++) {
               g = liczby[j] & 255;
               b = (liczby[j] >> 8) & 255;
               r = (liczby[j] >> 16) & 255;
               
               good = g > 0 && max(r, _r) + max(b, _b) + max(g-1, _g) < max_kraw;

               if (symmetric)
                 good = good || (g > 0 && max(b, _r) + max(r, _b) + max(g-1, _g) < max_kraw) ||    // b,r,g
                                (b > 0 && max(r, _r) + max(g, _b) + max(b-1, _g) < max_kraw) ||   // r,g,b
                                (r > 0 && max(b, _r) + max(g, _b) + max(r-1, _g) < max_kraw) ||   // b,g,r
                                (b > 0 && max(g, _r) + max(r, _b) + max(b-1, _g) < max_kraw) ||   // g,r,b
                                (r > 0 && max(g, _r) + max(b, _b) + max(r-1, _g) < max_kraw);      // g,b,r

               if (good) {
                  m++;
                  total_size += fsize2[i] * fsize[j];
                  fprintf(batch, "%s3b %d %s %d %s_%d_%d_%d_%d %s_%d_%d_%d_%d %d\n", ramsey_command, m, prefix, max_kraw-1, prefix3, _k, _r, _b, _g, prefix2, max_kraw, r, b, g, symmetric);
               }
             }
          }

          fclose(batch);
          printf("Ramrun %s %d. Tasks: %d. Total size: %.1f\n", p_new, max_kraw-1, m, total_size);

          if (split > 1) { // trzeba podzielić pracę na split kawałków 
            m = 0; i = 0; j = 0;
            n = 0; 
            c_size = 0;
            size = total_size/(split-n); //rozmiar +/- zadania w n-tym skrypcie
            sprintf(buf, "%s_%d_%d.%s", p_new, max_kraw-1, n, batch_extension);
            FILE *batch= fopen(buf, "wt");
            
            for (i = 0; (i < ile2) && (liczby2[i] >> 24 == max_kraw-1); i++) {
               _g = liczby2[i] & 255;
               _b = (liczby2[i] >> 8) & 255;
               _r = (liczby2[i] >> 16) & 255;

               for (j = 0; j < ile; j++) {
                 g = liczby[j] & 255;
                 b = (liczby[j] >> 8) & 255;
                 r = (liczby[j] >> 16) & 255;
                 k = liczby[j] >> 24;

                 good = g > 0 && max(r, _r) + max(b, _b) + max(g-1, _g) < max_kraw;

                 if (symmetric)
                   good = good || (g > 0 && max(b, _r) + max(r, _b) + max(g-1, _g) < max_kraw) ||    // b,r,g
                                  (b > 0 && max(r, _r) + max(g, _b) + max(b-1, _g) < max_kraw) ||   // r,g,b
                                  (r > 0 && max(b, _r) + max(g, _b) + max(r-1, _g) < max_kraw) ||   // b,g,r
                                  (b > 0 && max(g, _r) + max(r, _b) + max(b-1, _g) < max_kraw) ||   // g,r,b
                                  (r > 0 && max(g, _r) + max(b, _b) + max(r-1, _g) < max_kraw);      // g,b,r

                 if (good) {
                    if (size < 3*fsize2[i] * fsize[j]) {  // każdy plik ma co najmniej 3 zadania    
                      if (2*fsize2[i] > sqrt(size)) {  // dzielę i-ty plik (nieco chętniej)
                        // split file[i], na x +/- równych kawałków
                        x = (int)ceil(3*fsize2[i]/sqrt(size)); // jak już dzielę to na więcej kawałków
                        sprintf(fname, "%s_%d_%d_%d_%d", prefix3, max_kraw-1, _r, _b, _g);
                        sprintf(buf, "%s_0", fname);
                        if (stat(buf, &descript)) { // jeżeli nie było podziału to podziel
                          sprintf(buf, "%s -d -C %d %s %s_", split_command, (int)ceil(1024*fsize2[i]/x+120), fname, fname);
                          system(buf);

                          z = x > 10 ? 9 : x - 1;  // nazwy jednocyfrowe
                          for ( ; z >= 0; z--) {
                            sprintf(buf, "%s %s_0%d %s_%d", mv_command, fname, z, fname, z);
                            system(buf);
                          }
                          printf("%d %s split into %d pieces.\n", m+1, fname, x);
                        }
                        else x = (int)ceil(fsize2[i]*1024/descript.st_size);
                      }
                      else x = 1;
                      if (2*fsize[j] > sqrt(size)) { //dzielę j-ty plik
                        // split file[j], na y +/- równych kawałków
                        y = (int)ceil(3*fsize[j]/sqrt(size)); // jak już dzielę to na więcej kawałków
                        sprintf(fname, "%s_%d_%d_%d_%d", prefix2, k, r, b, g);
                        sprintf(buf, "%s_0", fname);
                        if (stat(buf, &descript)) { // jeżeli nie było podziału to podziel
                          sprintf(buf, "%s -d -C %d %s %s_", split_command, (int)ceil(1024*fsize[j]/y+120), fname, fname);
                          system(buf);

                          z = y > 10 ? 9 : y - 1;  // nazwy jednocyfrowe
                          for ( ; z >= 0; z--) {
                            sprintf(buf, "%s %s_0%d %s_%d", mv_command, fname, z, fname, z);
                            system(buf);
                          }
                          printf("%d %s split into %d pieces.\n", m+1, fname, y);
                        }
                        else y = (int)ceil(fsize[j]*1024/descript.st_size);
                      }
                      else y = 1;

                      // x - i-tych, y -j-tych plików
                      t = fsize2[i]/x * fsize[j]/y;
                      for (w = 0; w < x; w++)
                        for (z = 0; z < y; z++) {
                          m++;
                          if (fabs(size-c_size) < fabs(size - c_size - t)) { // nowy plik
                            fclose(batch);
                            total_size -= c_size;        // tyle zostało do zapisania
                            printf("File no %d, up to: %d, work-size: %.1f\n", n, m-1, c_size);
                            n++; c_size = 0;
                            size = total_size/(split-n); //rozmiar +/- zadania w n-tym skrypcie
                            sprintf(buf, "%s_%d_%d.%s", p_new, max_kraw-1, n, batch_extension);
                            FILE *batch= fopen(buf, "wt");
                          }
                          c_size += t; 
                          if (x == 1) fprintf(batch, "%s3b %d %s %d %s_%d_%d_%d_%d %s_%d %d\n", ramsey_command, m, prefix, max_kraw-1, prefix3, max_kraw-1, _r, _b, _g, fname, z, symmetric);
                          else if (y == 1) fprintf(batch, "%s3b %d %s %d %s_%d %s_%d_%d_%d_%d %d\n", ramsey_command, m, prefix, max_kraw-1, fname, w, prefix2, k, r, b, g, symmetric);
                          else fprintf(batch, "%s3b %d %s %d %s_%d_%d_%d_%d_%d %s_%d %d\n", ramsey_command, m, prefix, max_kraw-1, prefix3, max_kraw-1, _r, _b, _g, w, fname, z, symmetric);
                        }
                      
                    }
                    else {
                      m++;
                      if (fabs(size-c_size) < fabs(size - c_size - fsize2[i]*fsize[j])) { // nowy plik
                        fclose(batch);
                        total_size -= c_size;        // tyle zostało do zapisania
                        printf("File no %d, up to: %d, work-size: %.1f\n", n, m-1, c_size);
                        n++; c_size = 0;
                        size = total_size/(split-n); //rozmiar +/- zadania w n-tym skrypcie
                        sprintf(buf, "%s_%d_%d.%s", p_new, max_kraw-1, n, batch_extension);
                        FILE *batch= fopen(buf, "wt");
                      }
                      c_size += fsize2[i] * fsize[j];
                      fprintf(batch, "%s3b %d %s %d %s_%d_%d_%d_%d %s_%d_%d_%d_%d %d\n", ramsey_command, m, prefix, max_kraw-1, prefix3, max_kraw-1, _r, _b, _g, prefix2, k, r, b, g, symmetric);
                    }
                 }
               }
            }
            // i pozostaje takie samo
            for ( ; i < ile2; i++) {
               _g = liczby2[i] & 255;
               _b = (liczby2[i] >> 8) & 255;
               _r = (liczby2[i] >> 16) & 255;
               _k = liczby2[i] >> 24;

               for (j = 0; j < ile && (liczby[j] >> 24 == max_kraw); j++) {
                 g = liczby[j] & 255;
                 b = (liczby[j] >> 8) & 255;
                 r = (liczby[j] >> 16) & 255;

                 good = g > 0 && max(r, _r) + max(b, _b) + max(g-1, _g) < max_kraw;

                 if (symmetric)
                   good = good || (g > 0 && max(b, _r) + max(r, _b) + max(g-1, _g) < max_kraw) ||    // b,r,g
                                  (b > 0 && max(r, _r) + max(g, _b) + max(b-1, _g) < max_kraw) ||   // r,g,b
                                  (r > 0 && max(b, _r) + max(g, _b) + max(r-1, _g) < max_kraw) ||   // b,g,r
                                  (b > 0 && max(g, _r) + max(r, _b) + max(b-1, _g) < max_kraw) ||   // g,r,b
                                  (r > 0 && max(g, _r) + max(b, _b) + max(r-1, _g) < max_kraw);      // g,b,r

                 if (good) {
                    if (size < 3*fsize2[i] * fsize[j]) {  // każdy plik ma co najmniej 3 zadania    
                      if (2*fsize2[i] > sqrt(size)) {  // dzielę i-ty plik (nieco chętniej)
                        // split file[i], na x +/- równych kawałków
                        x = (int)ceil(3*fsize2[i]/sqrt(size)); // jak już dzielę to na więcej kawałków
                        sprintf(fname, "%s_%d_%d_%d_%d", prefix3, _k, _r, _b, _g);
                        sprintf(buf, "%s_0", fname);
                        if (stat(buf, &descript)) { // jeżeli nie było podziału to podziel
                          sprintf(buf, "%s -d -C %d %s %s_", split_command, (int)ceil(1024*fsize2[i]/x+120), fname, fname);
                          system(buf);

                          z = x > 10 ? 9 : x - 1;  // nazwy jednocyfrowe
                          for ( ; z >= 0; z--) {
                            sprintf(buf, "%s %s_0%d %s_%d", mv_command, fname, z, fname, z);
                            system(buf);
                          }
                          printf("%d %s split into %d pieces.\n", m+1, fname, x);
                        }
                        else x = (int)ceil(fsize2[i]*1024/descript.st_size);
                      }
                      else x = 1;
                      if (2*fsize[j] > sqrt(size)) { //dzielę j-ty plik
                        // split file[j], na y +/- równych kawałków
                        y = (int)ceil(3*fsize[j]/sqrt(size)); // jak już dzielę to na więcej kawałków
                        sprintf(fname, "%s_%d_%d_%d_%d", prefix2, max_kraw, r, b, g);
                        sprintf(buf, "%s_0", fname);
                        if (stat(buf, &descript)) { // jeżeli nie było podziału to podziel
                          sprintf(buf, "%s -d -C %d %s %s_", split_command, (int)ceil(1024*fsize[j]/y+120), fname, fname);
                          system(buf);

                          z = y > 10 ? 9 : y - 1;  // nazwy jednocyfrowe
                          for ( ; z >= 0; z--) {
                            sprintf(buf, "%s %s_0%d %s_%d", mv_command, fname, z, fname, z);
                            system(buf);
                          }
                          printf("%d %s split into %d pieces.\n", m+1, fname, y);
                        }
                        else y = (int)ceil(fsize[j]*1024/descript.st_size);
                      }
                      else y = 1;

                      // x - i-tych, y -j-tych plików
                      t = fsize2[i]/x * fsize[j]/y;
                      for (w = 0; w < x; w++)
                        for (z = 0; z < y; z++) {
                          m++;
                          if (fabs(size-c_size) < fabs(size - c_size - t)) { // nowy plik
                            fclose(batch);
                            total_size -= c_size;        // tyle zostało do zapisania
                            printf("File no %d, up to: %d, work-size: %.1f\n", n, m-1, c_size);
                            n++; c_size = 0;
                            size = total_size/(split-n); //rozmiar +/- zadania w n-tym skrypcie
                            sprintf(buf, "%s_%d_%d.%s", p_new, max_kraw-1, n, batch_extension);
                            FILE *batch= fopen(buf, "wt");
                          }
                          c_size += t; 
                          if (x == 1) fprintf(batch, "%s3b %d %s %d %s_%d_%d_%d_%d %s_%d %d\n", ramsey_command, m, prefix, max_kraw-1, prefix3, _k, _r, _b, _g, fname, z, symmetric);
                          else if (y == 1) fprintf(batch, "%s3b %d %s %d %s_%d %s_%d_%d_%d_%d %d\n", ramsey_command, m, prefix, max_kraw-1, fname, w, prefix2, max_kraw, r, b, g, symmetric);
                          else fprintf(batch, "%s3b %d %s %d %s_%d_%d_%d_%d_%d %s_%d %d\n", ramsey_command, m, prefix, max_kraw-1, prefix3, _k, _r, _b, _g, w, fname, z, symmetric);
                        }
                      
                    }
                    else {
                      m++;
                      if (fabs(size-c_size) < fabs(size - c_size - fsize2[i]*fsize[j])) { // nowy plik
                        fclose(batch);
                        total_size -= c_size;        // tyle zostało do zapisania
                        printf("File no %d, up to: %d, work-size: %.1f\n", n, m-1, c_size);
                        n++; c_size = 0;
                        size = total_size/(split-n); //rozmiar +/- zadania w n-tym skrypcie
                        sprintf(buf, "%s_%d_%d.%s", p_new, max_kraw-1, n, batch_extension);
                        FILE *batch= fopen(buf, "wt");
                      }
                      c_size += fsize2[i] * fsize[j];
                      fprintf(batch, "%s3b %d %s %d %s_%d_%d_%d_%d %s_%d_%d_%d_%d %d\n", ramsey_command, m, prefix, max_kraw-1, prefix3, _k, _r, _b, _g, prefix2, max_kraw, r, b, g, symmetric);
                    }
                 }
               }
            }
            fclose(batch);
            printf("File no %d, up to: %d, work-size: %.1f\n", n, m, c_size);
          } 
        }
      }
  }
  return 0;
}
