///////////////////////////////////////////////////
// Ramsey On-line game
// Author: Przemek Gordinowicz 
//         Pawel Pralat 
///////////////////////////////////////////////////
//
//  Part of the main file (ramsey.cpp) including auxiliary functions for speed improvement
//      create_description(col_graph *g) - list those coloured edges that can be distinguish without creating isomorphic copy 
//      check_excess(...) - check whether configuration isn't superconfiguration of some other considered before
//
///////////////////////////////////////////////////
void create_description(col_graph *g) {  // create description of coloured edges (removes isomorphic copies)
  graph_create2++;
  if (!g->descr) {
    int i, j, m;
    char *p, *cdstr;
    void *dict = NULL;

    i = g->edges[2] + g->edges[3];
    g->descr = new char[2*i+1];
    p = g->descr;

    for (i = 0; i < g->n; i++)
      for (j = i+1; j < g->n; j++)
        if (g->g[i][j] > NON_EDGE) {
          graph_create++;
          used[i] = 1;
          used[j] = 1;
          cdstr = g->col_graph_to_graph(0);
          m = mem_avail;
          inserttext(&dict, cdstr, -1);
          FREES(cdstr);
          if (mem_avail < m) { // isomorphs not yet considered
            *(p++) = '0'+ (char)i;
            *(p++) = '0'+ (char)j;
          }
          else graph_stop++;
          used[i] = 0;
          used[j] = 0;
        }
    *p = 0;
    free_dict(dict);
  }
}

int check_excess(col_graph *g, int kr = 0, int kb = 0) { // check whether graph is not redundant i.e. after removing one edge there is no isomorphic copy
  // kr <= kb <= 5, kr <= 4 - which number is calculated (R(kr, kb))
  int i, j, k;
  char *cdstr;

  if (!search_dict(((void**)graph_dict)[1], pg_readg_line, -1)) return 0; // such graph was already considered 
  //Removing superconfiguration of configuration from 0-th or 1st iteration
  //maybe there is red K_kr, blue K_kb, or their merging?
  if (kr || kb) {
    int i1, i2, i3, i4, i5;
    
    if (kr) {
      for (i1 = 0; i1 < g->n; i1++)
        for (i2 = i1+1; i2 < g->n; i2++)
          if (g->g[i1][i2] == 2) // red
            if (kr == 2) return 0;
            else for (i3 = i2+1; i3 < g->n; i3++)
              if (g->g[i1][i3] == 2 && g->g[i2][i3] == 2)
                if (kr == 3) return 0;
                else for (i4 = i3+1; i4 < g->n; i4++)
                  if (g->g[i1][i4] == 2 && g->g[i2][i4] == 2 && g->g[i3][i4] == 2)
                    return 0;
    }
    if (kb) {
      for (i1 = 0; i1 < g->n; i1++)
        for (i2 = i1+1; i2 < g->n; i2++)
          if (g->g[i1][i2] == 3) // blue
            if (kb == 2) return 0;
            else for (i3 = i2+1; i3 < g->n; i3++)
              if (g->g[i1][i3] == 3 && g->g[i2][i3] == 3)
                if (kb == 3) return 0;
                else for (i4 = i3+1; i4 < g->n; i4++)
                  if (g->g[i1][i4] == 3 && g->g[i2][i4] == 3 && g->g[i3][i4] == 3)
                    if (kb == 4) return 0;
                    else for (i5 = i4+1; i5 < g->n; i5++)
                      if (g->g[i1][i5] == 3 && g->g[i2][i5] == 3 && g->g[i3][i5] == 3 && g->g[i4][i5] == 3)
                        return 0;
    }
    if (kr && kb) {
      // configuration from the first iteration 
      for (i1 = 0; i1 < g->n; i1++)
        for (i2 = i1+1; i2 < g->n; i2++)
          if (g->g[i1][i2] <= NON_EDGE) {
            int still_good = 0;
            if (kr == 2) still_good = 1;
            else for (i3 = 0; i3 < g->n; i3++)
              if (g->g[i1][i3] == 2 && g->g[i2][i3] == 2) // red
                if (kr == 3) still_good = 1;
                else for (i4 = i3+1; i4 < g->n; i4++)
                  if (g->g[i1][i4] == 2 && g->g[i2][i4] == 2 && g->g[i3][i4] == 2)
                    still_good = 1;
            if (still_good)
              if (kb == 2) return 0;
              else for (i3 = 0; i3 < g->n; i3++)
                if (g->g[i1][i3] == 3 && g->g[i2][i3] == 3)
                  if (kb == 3) return 0;
                  else for (i4 = i3+1; i4 < g->n; i4++)
                    if (g->g[i1][i4] == 3 && g->g[i2][i4] == 3 && g->g[i3][i4] == 3)
                      if (kb == 4) return 0;
                      else for (i5 = i4+1; i5 < g->n; i5++)
                        if (g->g[i1][i5] == 3 && g->g[i2][i5] == 3 && g->g[i3][i5] == 3 && g->g[i4][i5] == 3)
                          return 0;
          }
    }
  }
  if (!symmetric) {
    for (i = 0; i < g->n; i++)
      for (j = i+1; j < g->n; j++)
        if (g->g[i][j] && ((void**)graph_dict)[g->g[i][j]]) {
          k = g->g[i][j];
          g->g[i][j] = g->g[j][i] = 0;
          cdstr = g->col_graph_to_graph(g->n);
          g->g[i][j] = g->g[j][i] = k;
          if (!search_dict(((void**)graph_dict)[k], cdstr, -1) || ((k == 1) && !search_dict(((void**)graph_dict)[0], cdstr, -1))) {
            FREES(cdstr);
            return 0;
          }
          else FREES(cdstr);
        }
    return 1;
  }
  else {
    int m;
    g->calculate_degree();
    for (i = 0; i < g->n; i++)
      for (j = i+1; j < g->n; j++)
        if (g->g[i][j]) {
          k = g->g[i][j];
          g->g[i][j] = g->g[j][i] = 0;
          g->edges[k]--;
          cdstr = g->col_graph_to_graph(g->n);
          m = k;
          if (g->edges[2] <= g->edges[3]) { 
            char *cdstr_2 = col_graph(g).col_graph_to_graph(g->n);
            if (g->edges[2] < g->edges[3] || strcmp(cdstr, cdstr_2) > 0) {
              FREES(cdstr);
              cdstr = cdstr_2;
              if (k == 2 && g->edges[2] < g->edges[3]) m = 3; // 2->3
            }
            else FREES(cdstr_2);
          }
          g->edges[k]++;
          g->g[i][j] = g->g[j][i] = k;
          if (!search_dict(((void**)graph_dict)[m], cdstr, -1) || ((m == 1) && !search_dict(((void**)graph_dict)[0], cdstr, -1))) {
            FREES(cdstr);
            return 0;
          }
          else FREES(cdstr);
        }
    return 1;
  }
}

