///////////////////////////////////////////////////
// Ramsey On-line game
// Author: Przemek Gordinowicz 
//         Pawel Pralat 
///////////////////////////////////////////////////
//
//  Part of the main file (ramsey.cpp) including functions for dictionary trees
//
///////////////////////////////////////////////////
// Dictionary trees that contains words (graphs in text nauty format) from 6-bit (64 el.) alphabet
// It is impossible that one word is a prefix or the second (according to nauty format prefix determines length)
//
// Structure:
//       - one node contains 8 void* pointers (possible branches) - corresponding to 3 bits of a letter 
//       - when 1st pointer (void *) points to itself then node has no branches - contains text - a suffix of a word
//       - variable mem_avail keeps info of memory available for a buffer
//////////////////////////////////////////////////

// puts a word when bit==-1 - elder part (3bits), bit != -1 - less significant part (3bits)
void inserttext(void *node, char *suff, char bit) {
  if (*((void **)node) == NULL) {
    size_t sz = strlen(suff) + 1 + sizeof(node);
    void *p = malloc(sz);
    mem_avail -= sz;

    *(void **)p = p;
    strcpy((char*)(((void**)p)+1), suff);
    *(void **)node = p; 
  }
  else if (*(void**)node == **(void***)node) {// ups - node with text/suffix 
    if (strcmp(suff, (char*)((*(void***)node)+1))) { // if suffixes are different go further, otherwise finish
      void *old = *(void **)node;
      char *suff2 = (char*)((void**)old+1);
      mem_avail += strlen(suff2) + 1 + sizeof(node); // we calculate that memory will be released in fact it happen after a while
      char bit1, bit2;

      do {
        void *p = malloc(node_size);
        mem_avail -= node_size;
        memset(p, 0, node_size);
        *(void **)node = p; 
        
        if (bit == -1) {
          bit1 = bit = ((*suff)-63) >> 3;
          bit2 = ((*suff2)-63) >> 3;
        }
        else {
          bit = -1;
          bit1 = ((*(suff++))-63) & 7;
          bit2 = ((*(suff2++))-63) & 7;
        }        
                 
        if (bit1 == bit2) node = (void *)(*((void***)node) + bit1);
      } while (bit1 == bit2);
      if (bit == -1) {
        inserttext((void *)(*((void***)node) + bit1), suff, -1);
        inserttext((void *)(*((void***)node) + bit2), suff2, -1);
      }
      else {
        inserttext((void *)(*((void***)node) + bit1), suff, bit1);
        inserttext((void *)(*((void***)node) + bit2), suff2, bit2);
      }
      free(old);
    }
  }
  else {
    if (bit == -1) inserttext((void *)(*((void***)node) + (((*suff)-63) >> 3)), suff, ((*suff)-63) >> 3);
    else inserttext((void *)(*((void***)node) + (((*suff)-63) & 7)), suff+1, -1);
  }
}

int search_dict(void *node, char *suff, char bit) {
  if (!node) return -1;
  if ((*(void **)node) == node)  // węzeł z tekstem
    return(strcmp(suff, (char*)(((void**)node)+1)));  
  else {
    if (bit == -1) return search_dict(*((void**)node + (((*suff)-63) >> 3)), suff, ((*suff)-63) >> 3);
    else return search_dict(*((void**)node + (((*suff)-63) & 7)), suff+1, -1);      
  }
}

void print_and_free(FILE *out, void *node, char* prefix, int n, char bit) {
  if ((*(void **)node) == node) { // node with text/suffix
    fprintf(out, "%s%s\n", prefix, (char*)(((void**)node)+1));
    mem_avail += strlen((char*)(((void**)node)+1)) + 1 + sizeof(node);
    free(node);
  }
  else {
    for (char i = 0; i < 8; i++)
      if (*((void**)node+i)) 
        if (n & 1) {
          prefix[n/2+1] = 0;
          prefix[n/2] = 63 + ((bit << 3) | i);
          print_and_free(out, *((void**)node+i), prefix, n+1, 0);
          prefix[n/2] = 0;
        }
        else print_and_free(out, *((void**)node+i), prefix, n+1, i);
    prefix[n] = 0;
    mem_avail += node_size;
    free(node);
  }
  return;
}

void free_dict(void *node) {
  if ((*(void **)node) == node) { // node with text
    mem_avail += strlen((char*)(((void**)node)+1)) + 1 + sizeof(node);
    free(node);
  }
  else {
    for (char i = 0; i < 8; i++)
      if (*((void**)node+i)) 
        free_dict(*((void**)node+i));
    mem_avail += node_size;
    free(node);
  }
  return;
}

// write dictionary from a tree to a file (graph_dict...)
void save_dictionary(void) {
  int i, j;
  char fname[100];
  char buf[1000];
  FILE *out;

  for (i = 0; i <= max_edges; i++)
    for (j = 0; j <= max_edges; j++) 
      if (((void **)graph_dict)[i*(max_edges+1)+j]) {
        sprintf(fname, "%s %d_%d_%d >>%s.txt", echo_command, max_edges, i, j, fname_prefix);
        system(fname);
        sprintf(fname, "%s_%d_%d_%d_%s_%d", fname_prefix, max_edges, i, j, fname_suffix, files_count[i*(max_edges+1)+j]++);
        out = fopen(fname, "wt");
        buf[0] = 0;
        print_and_free(out, ((void **)graph_dict)[i*(max_edges+1)+j], buf, 0, 0);        
        ((void **)graph_dict)[i*(max_edges+1)+j] = NULL;

        fclose(out);
      }
}

